@@ -63,7 +63,7 @@ function [oo_, options_mom_, M_] = run(bayestopt_, options_, oo_, estim_params_,
 %  o set_all_parameters.m
 %  o test_for_deep_parameters_calibration.m
 % =========================================================================
-% Copyright (C) 2020-2021 Dynare Team
+% Copyright (C) 2020-2022 Dynare Team
 %
 % This file is part of Dynare.
 %
@@ -121,7 +121,7 @@ if isempty(estim_params_) % structure storing the info about estimated parameter
         error('method_of_moments: The ''estimated_params'' block must not be empty')
     end
 end
-if isempty(M_.matched_moments) % structure storing the moments used for the method of moments estimation
+if ~isfield(M_,'matched_moments') || isempty(M_.matched_moments) % structure storing the moments used for the method of moments estimation
     error('method_of_moments: You need to provide a ''matched_moments'' block')
 end
 if ~isempty(bayestopt_) && any(bayestopt_.pshape==0) && any(bayestopt_.pshape~=0)
@@ -197,6 +197,7 @@ options_mom_ = set_default_option(options_mom_,'noprint',false);       % do not
 options_mom_ = set_default_option(options_mom_,'plot_priors',true);    % control plotting of priors
 options_mom_ = set_default_option(options_mom_,'prior_trunc',1e-10);   % probability of extreme values of the prior density that is ignored when computing bounds for the parameters
 options_mom_ = set_default_option(options_mom_,'TeX',false);           % print TeX tables and graphics
+options_mom_ = set_default_option(options_mom_,'verbosity',false);           % print TeX tables and graphics
 
 % Data and model options that can be set by the user in the mod file, otherwise default values are provided
 options_mom_ = set_default_option(options_mom_,'first_obs',1);     % number of first observation
@@ -205,6 +206,14 @@ options_mom_ = set_default_option(options_mom_,'nobs',NaN);        % number of o
 options_mom_ = set_default_option(options_mom_,'prefilter',false); % demean each data series by its empirical mean and use centered moments
 options_mom_ = set_default_option(options_mom_,'xls_sheet',1);     % name of sheet with data in Excel
 options_mom_ = set_default_option(options_mom_,'xls_range','');    % range of data in Excel sheet
+% temporary workaround for https://git.dynare.org/Dynare/dseries/-/issues/51
+if options_mom_.xls_sheet~=1
+    evalin('base','options_.xls_sheet=options_mom_.xls_sheet');
+end
+if ~isempty(options_mom_.xls_range)
+    evalin('base','options_.xls_range=options_mom_.xls_range');
+end
+
 % Recursive estimation and forecast are not supported
 if numel(options_mom_.nobs)>1
     error('method_of_moments: Recursive estimation and forecast for samples is not supported. Please set an integer as ''nobs''.');
@@ -363,7 +372,7 @@ analytic_jacobian_optimizers = [1, 3, 4, 13, 101]; %these are currently supporte
 options_mom_.initialize_estimated_parameters_with_the_prior_mode = 0; % needed by set_prior.m
 options_mom_.figures.textwidth = 0.8; %needed by plot_priors.m
 options_mom_.ramsey_policy = 0; % needed by evaluate_steady_state
-options_mom_.debug = false; %needed by resol.m
+options_mom_ = set_default_option(options_mom_,'debug',false); %neeeded by e.g. check_plot
 options_mom_.risky_steadystate = false; %needed by resol
 options_mom_.threads = options_.threads; %needed by resol
 options_mom_.jacobian_flag = true;
@@ -1008,11 +1017,7 @@ fprintf('\n==== Method of Moments Estimation (%s) Completed ====\n\n',options_mo
 % Step 9: Clean up
 % -------------------------------------------------------------------------
 %reset warning state
-if isoctave
-    warning('on')
-else
-    warning on
-end
+warning_config;
 
 if isoctave && isfield(options_, 'prior_restrictions') && ...
    isfield(options_.prior_restrictions, 'routine')
