@@ -1,5 +1,5 @@
 function [a, a1, P, P1, v, T, R, C, regimes_, error_flag, M_, lik, etahat] = kalman_update_algo_1(a,a1,P,P1,data_index,Z,v,Y,H,QQQ,T0,R0,TT,RR,CC,regimes0,M_,oo_,options_,occbin_options)
-% function [a, a1, P, P1, v, T, R, C, regimes_, info, M_, lik, etahat] = kalman_update_algo_1(a,a1,P,P1,data_index,Z,v,Y,H,QQQ,T0,R0,TT,RR,CC,regimes0,M_,oo_,options_,occbin_options)
+% function [a, a1, P, P1, v, T, R, C, regimes_, error_flag, M_, lik, etahat] = kalman_update_algo_1(a,a1,P,P1,data_index,Z,v,Y,H,QQQ,T0,R0,TT,RR,CC,regimes0,M_,oo_,options_,occbin_options)
 % INPUTS
 % - a               [N by 1]                t-1's state estimate
 % - a1              [N by 2]               state predictions made at t-1:t
@@ -42,7 +42,7 @@ function [a, a1, P, P1, v, T, R, C, regimes_, error_flag, M_, lik, etahat] = kal
 % Philipp Pfeiffer, Marco Ratto (2021), Efficient and robust inference of models with occasionally binding
 % constraints, Working Papers 2021-03, Joint Research Centre, European Commission 
 
-% Copyright (C) 2021 Dynare Team
+% Copyright (C) 2021-2022 Dynare Team
 %
 % This file is part of Dynare.
 %
@@ -61,6 +61,16 @@ function [a, a1, P, P1, v, T, R, C, regimes_, error_flag, M_, lik, etahat] = kal
 
 warning off
 
+regimes_(1).regime=false;
+regimes_(2).regime=false;
+regimes_(3).regime=false;
+regimes_(1).regimestart=NaN;
+regimes_(2).regimestart=NaN;
+regimes_(3).regimestart=NaN;
+R=NaN(size(RR));
+C=NaN(size(CC));
+lik=Inf;
+
 sto.a=a;
 sto.a1=a1;
 sto.P=P;
@@ -94,17 +104,21 @@ PZI         = P1(:,:,t)*ZZ'*iF(di,di,t);
 L(:,:,t)    = eye(mm)-PZI*ZZ;
 
 if ~options_.occbin.filter.use_relaxation
-    [a, a1, P, P1, v, alphahat, etahat, lik] = occbin_kalman_update0(a,a1,P,P1,data_index,Z,v,Y,H,QQQ,TT,RR,CC,iF,L,mm, options_.rescale_prediction_error_covariance, options_.occbin.likelihood.IF_likelihood);
+    [a, a1, P, P1, v, alphahat, etahat, lik, error_flag] = occbin_kalman_update0(a,a1,P,P1,data_index,Z,v,Y,H,QQQ,TT,RR,CC,iF,L,mm, options_.rescale_prediction_error_covariance, options_.occbin.likelihood.IF_likelihood);
 else
     [~,~,~,~,~,~, TTx, RRx, CCx] ...
         = occbin.dynare_resolve(M_,options_,oo_, base_regime,'reduced_state_space',T0,R0);
+    regimes0(1)=base_regime;
     TT(:,:,2) = TTx(:,:,end);
     RR(:,:,2) = RRx(:,:,end);
     CC(:,2) = CCx(:,end);
-    [a, a1, P, P1, v, alphahat, etahat, lik] = occbin_kalman_update0(a,a1,P,P1,data_index,Z,v,Y,H,QQQ,TT,RR,CC,iF,L,mm, options_.rescale_prediction_error_covariance, options_.occbin.likelihood.IF_likelihood);
-    regimes0(1)=base_regime;
+    [a, a1, P, P1, v, alphahat, etahat, lik, error_flag] = occbin_kalman_update0(a,a1,P,P1,data_index,Z,v,Y,H,QQQ,TT,RR,CC,iF,L,mm, options_.rescale_prediction_error_covariance, options_.occbin.likelihood.IF_likelihood);
+end
+if error_flag
+    etahat=NaN(size(QQQ,1),1);
+    T=NaN(size(TT));
+    return;
 end
-
 
 %% run here the occbin simul
 opts_simul = occbin_options.opts_simul;
@@ -121,7 +135,12 @@ else
     my_order_var = oo_.dr.order_var;
 end
 options_.occbin.simul=opts_simul;
+options_.noprint=1;
 [~, out, ss] = occbin.solver(M_,oo_,options_);
+if out.error_flag
+    error_flag = out.error_flag;
+    return;
+end
 
 regimes_ = out.regime_history;
 if M_.occbin.constraint_nbr==1
@@ -205,6 +224,10 @@ if any(myregime) || ~isequal(regimes_(1),regimes0(1))
         opts_simul.periods = max(opts_simul.periods,max(myregimestart));
         options_.occbin.simul=opts_simul;
         [~, out, ss] = occbin.solver(M_,oo_,options_);
+        if out.error_flag
+            error_flag = out.error_flag;
+            return;
+        end
         regimes0=regimes_;
         regimes_ = out.regime_history;
         if niter>1
@@ -250,6 +273,10 @@ if any(myregime) || ~isequal(regimes_(1),regimes0(1))
                     opts_simul.maxit=1;
                     options_.occbin.simul=opts_simul;
                     [~, out, ss] = occbin.solver(M_,oo_,options_);
+                    if out.error_flag
+                        error_flag = out.error_flag;
+                        return;
+                    end
                 end
             end
         end
@@ -303,6 +330,10 @@ if ~error_flag && niter>options_.occbin.likelihood.max_number_of_iterations && ~
             opts_simul.periods = max(opts_simul.periods,max(myregimestart));
             options_.occbin.simul=opts_simul;
             [~, out, ss] = occbin.solver(M_,oo_,options_);
+            if out.error_flag
+                error_flag = out.error_flag;
+                return;
+            end
             if isequal(out.regime_history(1),regimes_(1))
                 error_flag=0;
                 break
@@ -323,10 +354,15 @@ P(:,:,2) = T(:,:,2)*P(:,:,1)*transpose(T(:,:,2))+QQ;
 regimes_=regimes_(1:3);
 etahat=etahat(:,2);
 
-warning on
+warning_config;
 end
 
-function [a, a1, P, P1, v, alphahat, etahat, lik] = occbin_kalman_update0(a,a1,P,P1,data_index,Z,v,Y,H,QQQ,TT,RR,CC,iF,L,mm, rescale_prediction_error_covariance, IF_likelihood)
+function [a, a1, P, P1, v, alphahat, etahat, lik, error_flag] = occbin_kalman_update0(a,a1,P,P1,data_index,Z,v,Y,H,QQQ,TT,RR,CC,iF,L,mm, rescale_prediction_error_covariance, IF_likelihood)
+alphahat=NaN(size(a));
+etahat=NaN(size(QQQ,1),2);
+lik=Inf; 
+error_flag=0;
+
 warning off
 if nargin<18
     IF_likelihood=0;
@@ -354,6 +390,10 @@ else
     v(di,t)      = Y(di,t) - ZZ*a(:,t);
     F = ZZ*P(:,:,t)*ZZ' + H(di,di);
     sig=sqrt(diag(F));
+    if any(any(isnan(F)))
+        error_flag=1;
+        return;
+    end
     if rank(F)<size(F,1) 
         % here we trap cases when some OBC regime triggers singularity 
         % e.g. no shock to interest rate at ZLB
@@ -424,5 +464,5 @@ while t > 1
     end
 end
 
-warning on
+warning_config;
 end
