@@ -205,7 +205,9 @@ if fload==0
             %         end
             %       end
             for j=1:np
-                lpmat(:,j)=lpmat(:,j).*(bounds.ub(j+nshock)-bounds.lb(j+nshock))+bounds.lb(j+nshock);
+                lower_bound=max(-options_.huge_number,bounds.lb(j+nshock));
+                upper_bound=min(options_.huge_number,bounds.ub(j+nshock));
+                lpmat(:,j)=lpmat(:,j).*(upper_bound-lower_bound)+lower_bound;
             end
         else
             xx=prior_draw_gsa(0,[lpmat0 lpmat]);
@@ -271,7 +273,7 @@ if fload==0
             for j=1:np
                 ub=xparam1(j+nshock)*(1+sign(xparam1(j+nshock))*neighborhood_width);
                 lb=xparam1(j+nshock)*(1-sign(xparam1(j+nshock))*neighborhood_width);
-                if bounds.ub(j+nshock)>=xparam1(j) && bounds.lb(j)<=xparam1(j+nshock)
+                if bounds.ub(j+nshock)>=xparam1(j+nshock) && bounds.lb(j+nshock)<=xparam1(j+nshock)
                     ub=min([bounds.ub(j+nshock) ub]);
                     lb=max([bounds.lb(j+nshock) lb]);
                 else
