@@ -201,7 +201,7 @@ ModelTree::operator=(const ModelTree &m)
 }
 
 bool
-ModelTree::computeNormalization(const jacob_map_t &contemporaneous_jacobian, bool verbose)
+ModelTree::computeNormalization(const jacob_map_t &contemporaneous_jacobian, bool dynamic, bool verbose)
 {
   const int n = equations.size();
 
@@ -241,7 +241,7 @@ ModelTree::computeNormalization(const jacob_map_t &contemporaneous_jacobian, boo
       it != mate_map.begin() + n)
     {
       if (verbose)
-        cerr << "ERROR: Could not normalize the model. Variable "
+        cerr << "ERROR: Could not normalize the " << (dynamic ? "dynamic" : "static") << " model. Variable "
              << symbol_table.getName(symbol_table.getID(SymbolType::endogenous, it - mate_map.begin()))
              << " is not in the maximum cardinality matching." << endl;
       check = false;
@@ -250,9 +250,9 @@ ModelTree::computeNormalization(const jacob_map_t &contemporaneous_jacobian, boo
 }
 
 void
-ModelTree::computeNonSingularNormalization(const jacob_map_t &contemporaneous_jacobian)
+ModelTree::computeNonSingularNormalization(const jacob_map_t &contemporaneous_jacobian, bool dynamic)
 {
-  cout << "Normalizing the model..." << endl;
+  cout << "Normalizing the " << (dynamic ? "dynamic" : "static") << " model..." << endl;
 
   int n = equations.size();
 
@@ -284,14 +284,14 @@ ModelTree::computeNonSingularNormalization(const jacob_map_t &contemporaneous_ja
           suppressed++;
 
       if (suppressed != last_suppressed)
-        found_normalization = computeNormalization(normalized_contemporaneous_jacobian_above_cutoff, false);
+        found_normalization = computeNormalization(normalized_contemporaneous_jacobian_above_cutoff, dynamic, false);
       last_suppressed = suppressed;
       if (!found_normalization)
         {
           current_cutoff /= 2;
           // In this last case try to normalize with the complete jacobian
           if (current_cutoff <= cutoff_lower_limit)
-            found_normalization = computeNormalization(normalized_contemporaneous_jacobian, false);
+            found_normalization = computeNormalization(normalized_contemporaneous_jacobian, dynamic, false);
         }
     }
 
@@ -302,12 +302,19 @@ ModelTree::computeNonSingularNormalization(const jacob_map_t &contemporaneous_ja
          normalization even with a potential singularity.
          TODO: Explain why symbolic_jacobian is not contemporaneous. */
       auto symbolic_jacobian = computeSymbolicJacobian();
-      found_normalization = computeNormalization(symbolic_jacobian, true);
+      found_normalization = computeNormalization(symbolic_jacobian, dynamic, true);
     }
 
   if (!found_normalization)
     {
-      cerr << "No normalization could be computed. Aborting." << endl;
+      /* Some models don’t have a steady state, and this can cause the
+         normalization to fail (e.g. if some variable only appears in a diff(),
+         it will disappear from the static model). Suggest the “no_static”
+         option as a possible solution. */
+      if (!dynamic)
+        cerr << "If your model does not have a steady state, you may want to try the 'no_static' option of the 'model' block." << endl;
+      /* The last call to computeNormalization(), which was verbose, already
+         printed an error message, so we can immediately exit. */
       exit(EXIT_FAILURE);
     }
 }
@@ -818,7 +825,9 @@ ModelTree::determineLinearBlocks()
   for (int blk = 0; blk < static_cast<int>(blocks.size()); blk++)
     switch (blocks[blk].simulation_type)
       {
+      case BlockSimulationType::solveBackwardSimple:
       case BlockSimulationType::solveBackwardComplete:
+      case BlockSimulationType::solveForwardSimple:
       case BlockSimulationType::solveForwardComplete:
         for (const auto &[indices, d1] : blocks_derivatives[blk])
           {
@@ -1254,10 +1263,9 @@ ModelTree::testNestedParenthesis(const string &str) const
 }
 
 void
-ModelTree::compileTemporaryTerms(ostream &code_file, unsigned int &instruction_number, bool dynamic, bool steady_dynamic, temporary_terms_t &temporary_terms_union, const temporary_terms_idxs_t &temporary_terms_idxs) const
+ModelTree::compileTemporaryTerms(ostream &code_file, unsigned int &instruction_number, bool dynamic, bool steady_dynamic, temporary_terms_t &temporary_terms_union, const temporary_terms_idxs_t &temporary_terms_idxs, deriv_node_temp_terms_t &tef_terms) const
 {
   // To store the functions that have already been written in the form TEF* = ext_fun();
-  deriv_node_temp_terms_t tef_terms;
   for (auto [tt, idx] : temporary_terms_idxs)
     {
       if (dynamic_cast<AbstractExternalFunctionNode *>(tt))
@@ -1392,7 +1400,7 @@ ModelTree::writeModelEquations(ostream &output, ExprNodeOutputType output_type,
 }
 
 void
-ModelTree::compileModelEquations(ostream &code_file, unsigned int &instruction_number, bool dynamic, bool steady_dynamic, const temporary_terms_t &temporary_terms_union, const temporary_terms_idxs_t &temporary_terms_idxs) const
+ModelTree::compileModelEquations(ostream &code_file, unsigned int &instruction_number, bool dynamic, bool steady_dynamic, const temporary_terms_t &temporary_terms_union, const temporary_terms_idxs_t &temporary_terms_idxs, const deriv_node_temp_terms_t &tef_terms) const
 {
   for (int eq = 0; eq < static_cast<int>(equations.size()); eq++)
     {
@@ -1413,8 +1421,8 @@ ModelTree::compileModelEquations(ostream &code_file, unsigned int &instruction_n
 
       if (vrhs != 0) // The right hand side of the equation is not empty ==> residual=lhs-rhs;
         {
-          lhs->compile(code_file, instruction_number, false, temporary_terms_union, temporary_terms_idxs, dynamic, steady_dynamic);
-          rhs->compile(code_file, instruction_number, false, temporary_terms_union, temporary_terms_idxs, dynamic, steady_dynamic);
+          lhs->compile(code_file, instruction_number, false, temporary_terms_union, temporary_terms_idxs, dynamic, steady_dynamic, tef_terms);
+          rhs->compile(code_file, instruction_number, false, temporary_terms_union, temporary_terms_idxs, dynamic, steady_dynamic, tef_terms);
 
           FBINARY_ fbinary{static_cast<int>(BinaryOpcode::minus)};
           fbinary.write(code_file, instruction_number);
@@ -1424,7 +1432,7 @@ ModelTree::compileModelEquations(ostream &code_file, unsigned int &instruction_n
         }
       else // The right hand side of the equation is empty ==> residual=lhs;
         {
-          lhs->compile(code_file, instruction_number, false, temporary_terms_union, temporary_terms_idxs, dynamic, steady_dynamic);
+          lhs->compile(code_file, instruction_number, false, temporary_terms_union, temporary_terms_idxs, dynamic, steady_dynamic, tef_terms);
           FSTPR_ fstpr(eq);
           fstpr.write(code_file, instruction_number);
         }
@@ -1821,21 +1829,7 @@ ModelTree::writeJsonModelEquations(ostream &output, bool residuals) const
 
           output << R"(, "rhs": ")";
           rhs->writeJsonOutput(output, {}, {});
-          output << R"(")";
-          try
-            {
-              // Test if the right hand side of the equation is empty.
-              if (rhs->eval({}) != 0)
-                {
-                  output << R"(, "rhs": ")";
-                  rhs->writeJsonOutput(output, {}, {});
-                  output << R"(")";
-                }
-            }
-          catch (ExprNode::EvalException &e)
-            {
-            }
-          output << "}";
+          output << R"("})";
         }
       else
         {
